/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Bonneau and Obeo - initial API and implementation
 */

package fr.obeo.acceleo.chain.ui.debug;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.ui.DebugUITools;
import org.eclipse.debug.ui.ILaunchShortcut;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorPart;

/**
 * A launch shortcut is capable of launching a chain file in the workbench.
 * 
 * @author David Bonneau
 * 
 */
public class AcceleoLaunchShortcut implements ILaunchShortcut {

	private static final String ID_ACCELEO_CONFIGURATION_TYPE = "fr.obeo.acceleo.chain.ui.debug.launchConfigurationType";

	/* (non-Javadoc) */
	public void launch(ISelection selection, String mode) {
		// must be a structured selection with one file selected
		IFile file = (IFile) ((IStructuredSelection) selection).getFirstElement();
		// check for an existing launch config for the pda file
		String path = file.getFullPath().toString();
		ILaunchManager launchManager = DebugPlugin.getDefault().getLaunchManager();
		ILaunchConfigurationType type = launchManager.getLaunchConfigurationType(ID_ACCELEO_CONFIGURATION_TYPE);
		try {
			ILaunchConfiguration[] configurations = launchManager.getLaunchConfigurations(type);
			for (int i = 0; i < configurations.length; i++) {
				ILaunchConfiguration configuration = configurations[i];
				String attribute = configuration.getAttribute(AcceleoLaunchDelegate.CHAIN_ATTRIBUTE_ID, (String) null);
				if (path.equals(attribute)) {
					DebugUITools.launch(configuration, mode);
					return;
				}
			}
		} catch (CoreException e) {
			return;
		}

		try {
			// create a new configuration for the pda file
			ILaunchConfigurationWorkingCopy workingCopy = type.newInstance(null, file.getName());
			workingCopy.setAttribute(AcceleoLaunchDelegate.CHAIN_ATTRIBUTE_ID, path);
			ILaunchConfiguration configuration = workingCopy.doSave();
			DebugUITools.launch(configuration, mode);
		} catch (CoreException e1) {
		}

	}

	/* (non-Javadoc) */
	public void launch(IEditorPart editor, String mode) {
	}

}
