/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.chain.ui.wizards;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;

import fr.obeo.acceleo.chain.ActionSet;
import fr.obeo.acceleo.chain.Chain;
import fr.obeo.acceleo.chain.ChainFactory;
import fr.obeo.acceleo.chain.EmfMetamodel;
import fr.obeo.acceleo.chain.Folder;
import fr.obeo.acceleo.chain.Generate;
import fr.obeo.acceleo.chain.Generator;
import fr.obeo.acceleo.chain.Log;
import fr.obeo.acceleo.chain.Model;
import fr.obeo.acceleo.chain.Repository;
import fr.obeo.acceleo.ecore.factories.EFactory;
import fr.obeo.acceleo.ecore.factories.FactoryException;
import fr.obeo.acceleo.gen.template.scripts.SpecificScript;
import fr.obeo.acceleo.gen.ui.wizards.AcceleoNewFileWizardPage;
import fr.obeo.acceleo.gen.ui.wizards.AcceleoSelectFilesWizardPage;
import fr.obeo.acceleo.gen.ui.wizards.AcceleoSelectMetamodelWizardPage;

/**
 * The role of this wizard is to create a chain.
 * 
 * @author www.obeo.fr
 */
public class AcceleoNewChainWizard extends AcceleoNewEmptyChainWizard {

	/**
	 * The page to select the metamodel.
	 */
	protected AcceleoSelectMetamodelWizardPage pageSelectMetamodel;

	private String metamodelURI;

	/**
	 * The page to select the model.
	 */
	protected AcceleoSelectFilesWizardPage pageSelectModel;

	private String modelPath;

	/**
	 * The page to select the scripts.
	 */
	protected AcceleoSelectFilesWizardPage pageSelectScripts;

	private IPath[] scripts;

	/**
	 * Constructor.
	 */
	public AcceleoNewChainWizard() {
		super();
	}

	/* (non-Javadoc) */
	public void addPages() {
		addPage(pageSelectMetamodel = new AcceleoSelectMetamodelWizardPage("Select the metamodel (1/4)", new String[] { "ecore" }));
		addPage(pageSelectModel = new AcceleoSelectFilesWizardPage("Select the model (2/4)", 1, 1, new String[] { "*" }, false));
		addPage(pageSelectScripts = new AcceleoSelectFilesWizardPage("Select the templates (3/4)", 1, -1, new String[] { SpecificScript.GENERATORS_EXTENSION }, true));
		addPage(pageNewFile = new AcceleoNewFileWizardPage("Create a new chain (4/4)", selection, "chain"));
	}

	/* (non-Javadoc) */
	public boolean performFinish() {
		metamodelURI = pageSelectMetamodel.getMetamodelURI();
		modelPath = pageSelectModel.getSelection()[0].toString();
		scripts = pageSelectScripts.getSelection();
		return super.performFinish();
	}

	/* (non-Javadoc) */
	protected Object createChainContent(IPath path) throws FactoryException, CoreException {
		// Create root element
		Chain chain = ChainFactory.eINSTANCE.createChain();
		// Repository
		Repository repository = ChainFactory.eINSTANCE.createRepository();
		EFactory.eSet(chain, "repository", repository);
		// Action Set
		ActionSet actionSet = ChainFactory.eINSTANCE.createActionSet();
		EFactory.eAdd(chain, "actions", actionSet);
		// Model file
		Model model = ChainFactory.eINSTANCE.createModel();
		EFactory.eAdd(repository, "files", model);
		EFactory.eSet(model, "path", modelPath);
		// Target folder
		Folder folder = ChainFactory.eINSTANCE.createFolder();
		EFactory.eAdd(repository, "files", folder);
		EFactory.eSet(folder, "path", path.removeLastSegments(1).toString());
		// Log
		Log log = ChainFactory.eINSTANCE.createLog();
		EFactory.eAdd(repository, "files", log);
		EFactory.eSet(log, "path", path.removeLastSegments(1).append(path.removeFileExtension().lastSegment() + ".log.txt").toString());
		// Metamodel file
		EmfMetamodel pim = ChainFactory.eINSTANCE.createEmfMetamodel();
		EFactory.eAdd(repository, "files", pim);
		EFactory.eSet(pim, "path", metamodelURI);
		// Script files and generate actions
		for (int i = 0; i < scripts.length; i++) {
			Generator script = ChainFactory.eINSTANCE.createGenerator();
			EFactory.eAdd(repository, "files", script);
			EFactory.eSet(script, "path", scripts[i].toString());
			Generate action = ChainFactory.eINSTANCE.createGenerate();
			EFactory.eAdd(actionSet, "actions", action);
			EFactory.eSet(action, "documentation", scripts[i].removeFileExtension().lastSegment());
			EFactory.eSet(action, "folder", folder);
			EFactory.eSet(action, "log", log);
			EFactory.eSet(action, "metamodel", pim);
			EFactory.eSet(action, "model", model);
			EFactory.eSet(action, "generator", script);
		}
		return chain;
	}

}
