/*
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Bonneau and Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.debug.ui.actions;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.text.source.IVerticalRulerInfo;
import org.eclipse.ui.texteditor.ITextEditor;

import fr.obeo.acceleo.gen.debug.ui.AcceleoGenDebugUiPlugin;
import fr.obeo.acceleo.gen.debug.ui.model.AcceleoLineBreakpoint;

/**
 * The breakpoint ruler action to set the condition.
 * 
 * @author www.obeo.fr
 * 
 */
public class BreakpointSelectModelRulerAction extends AbstractBreakpointRulerAction implements IAction {

	private static final String CONDITION_FILTER_ATTRIBUTE = "condition";

	/**
	 * The constructor.
	 * 
	 * @param editor
	 *            is the editor
	 * @param rulerInfo
	 *            is the vertical ruler
	 */
	public BreakpointSelectModelRulerAction(ITextEditor editor, IVerticalRulerInfo rulerInfo) {
		setInfo(rulerInfo);
		setTextEditor(editor);
		setText("Breakpoint Condition...");
	}

	/* (non-Javadoc) */
	public void run() {
		IBreakpoint breakPoint = getBreakpoint();
		if (breakPoint != null && breakPoint.getMarker() != null) {
			try {
				String value = (String) breakPoint.getMarker().getAttribute(CONDITION_FILTER_ATTRIBUTE, "");
				InputDialog dialog = new InputDialog(getTextEditor().getSite().getShell(), "Breakpoint Condition...", "Put an Acceleo expression to filter the objects to debug", value, null);
				if (dialog.open() == InputDialog.OK && dialog.getValue() != null) {
					breakPoint.getMarker().setAttribute(CONDITION_FILTER_ATTRIBUTE, dialog.getValue());
				}
			} catch (CoreException e) {
				AcceleoGenDebugUiPlugin.getDefault().log(e, true);
			}
		}
	}

	/* (non-Javadoc) */
	public void update() {
		setBreakpoint(determineBreakpoint());
		if (getBreakpoint() == null || !(getBreakpoint() instanceof AcceleoLineBreakpoint)) {
			setBreakpoint(null);
			setEnabled(false);
			return;
		}
		setEnabled(true);
	}

}