/*
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Bonneau and Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.debug.ui.model;

import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.model.IDebugTarget;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.IStreamsProxy;

/**
 * Process of an AcceloDebugTarget
 * 
 * @author David Bonneau
 * 
 */
public class AcceleoProcess extends AcceleoDebugElement implements IProcess {

	/**
	 * Constructor.
	 * 
	 * @param target
	 *            is the debug target
	 */
	public AcceleoProcess(AcceleoDebugTarget target) {
		super(target);
	}

	/* (non-Javadoc) */
	public String getAttribute(String key) {
		return getDebugTarget().getLaunch().getAttribute(key);
	}

	/* (non-Javadoc) */
	public int getExitValue() throws DebugException {
		return 0;
	}

	/* (non-Javadoc) */
	public String getLabel() {
		return "Acceleo generation";
	}

	/* (non-Javadoc) */
	public ILaunch getLaunch() {
		return getDebugTarget().getLaunch();
	}

	/* (non-Javadoc) */
	public IStreamsProxy getStreamsProxy() {
		return null;
	}

	/* (non-Javadoc) */
	public void setAttribute(String key, String value) {
		getDebugTarget().getLaunch().setAttribute(key, value);
	}

	/* (non-Javadoc) */
	public Object getAdapter(Class adapter) {
		Object ret = null;
		if (adapter.equals(IProcess.class)) {
			ret = this;
		}
		if (adapter.equals(IDebugTarget.class)) {
			ret = getLaunch().getDebugTarget();
		}
		if (adapter.equals(ILaunch.class)) {
			ret = getLaunch();
		}
		return ret;
	}

	/* (non-Javadoc) */
	public boolean canTerminate() {
		return true;
	}

	/* (non-Javadoc) */
	public boolean isTerminated() {
		return getDebugTarget().isTerminated();
	}

	/* (non-Javadoc) */
	public void terminate() throws DebugException {
		getDebugTarget().terminate();
	}

}
